var durationFormat = function(val) {
	if (val < 1000) {
		return "";
	}
	else if (val < 60000) {
		var r = val % 1000;
		return Math.round (val / 1000) + "s" + durationFormat (r);
	}
	else if (val < 3600000) {
		var r = val % 60000;
		return Math.round (val / 60000) + "m " + durationFormat (r);
	}
	else if (val < 86400000) {
		var r = val % 3600000;
		return Math.round (val / 3600000) + "h " + durationFormat (r);
	}
	else if (val < 604800000) {
		var r = val % 86400000;
		return Math.round (val / 86400000) + "d " + durationFormat (r);
	}
	else if (val < 2419200000) {
		var r = val % 604800000;
		return Math.round (val / 604800000) + "w " + durationFormat (r);
	}
	else if (val < 29030400000) {
		var r = val % 2419200000;
		return Math.round (val / 2419200000) + "M " + durationFormat (r);
	}
	else {
		return val;
	}
};

var dateFormat = function() {
		var token = /d{1,4}|m{1,4}|yy(?:yy)?|([HhMsTt])\1?|[LloSZ]|"[^"]*"|'[^']*'/g,
			timezone = /\b(?:[PMCEA][SDP]T|(?:Pacific|Mountain|Central|Eastern|Atlantic) (?:Standard|Daylight|Prevailing) Time|(?:GMT|UTC)(?:[-+]\d{4})?)\b/g,
			timezoneClip = /[^-+\dA-Z]/g,
			pad = function(val, len) {
				val = String(val);
				len = len || 2;
				while (val.length < len) val = "0" + val;
				return val;
			};

		// Regexes and supporting functions are cached through closure
		return function(date, mask, utc) {
			var dF = dateFormat;

			// You can't provide utc if you skip other args (use the "UTC:" mask prefix)
			if (arguments.length == 1 && Object.prototype.toString.call(date) == "[object String]" && !/\d/.test(date)) {
				mask = date;
				date = undefined;
			}

			// Passing date through Date applies Date.parse, if necessary
			date = date ? new Date(date) : new Date;
			if (isNaN(date)) throw SyntaxError("invalid date");

			mask = String(dF.masks[mask] || mask || dF.masks["default"]);

			// Allow setting the utc argument via the mask
			if (mask.slice(0, 4) == "UTC:") {
				mask = mask.slice(4);
				utc = true;
			}

			var _ = utc ? "getUTC" : "get",
				d = date[_ + "Date"](),
				D = date[_ + "Day"](),
				m = date[_ + "Month"](),
				y = date[_ + "FullYear"](),
				H = date[_ + "Hours"](),
				M = date[_ + "Minutes"](),
				s = date[_ + "Seconds"](),
				L = date[_ + "Milliseconds"](),
				o = utc ? 0 : date.getTimezoneOffset(),
				flags = {
					d: d,
					dd: pad(d),
					ddd: dF.i18n.dayNames[D],
					dddd: dF.i18n.dayNames[D + 7],
					m: m + 1,
					mm: pad(m + 1),
					mmm: dF.i18n.monthNames[m],
					mmmm: dF.i18n.monthNames[m + 12],
					yy: String(y).slice(2),
					yyyy: y,
					h: H % 12 || 12,
					hh: pad(H % 12 || 12),
					H: H,
					HH: pad(H),
					M: M,
					MM: pad(M),
					s: s,
					ss: pad(s),
					l: pad(L, 3),
					L: pad(L > 99 ? Math.round(L / 10) : L),
					t: H < 12 ? "a" : "p",
					tt: H < 12 ? "am" : "pm",
					T: H < 12 ? "A" : "P",
					TT: H < 12 ? "AM" : "PM",
					Z: utc ? "UTC" : (String(date).match(timezone) || [""]).pop().replace(timezoneClip, ""),
					o: (o > 0 ? "-" : "+") + pad(Math.floor(Math.abs(o) / 60) * 100 + Math.abs(o) % 60, 4),
					S: ["th", "st", "nd", "rd"][d % 10 > 3 ? 0 : (d % 100 - d % 10 != 10) * d % 10]
				};

			return mask.replace(token, function($0) {
				return $0 in flags ? flags[$0] : $0.slice(1, $0.length - 1);
			});
		};
	}();

// Some common format strings
dateFormat.masks = {
	"default": "ddd mmm dd yyyy HH:MM:ss",
	shortDate: "m/d/yy",
	mediumDate: "mmm d, yyyy",
	longDate: "mmmm d, yyyy",
	fullDate: "dddd, mmmm d, yyyy",
	shortTime: "h:MM TT",
	mediumTime: "h:MM:ss TT",
	longTime: "h:MM:ss TT Z",
	isoDate: "yyyy-mm-dd",
	isoTime: "HH:MM:ss",
	isoDateTime: "yyyy-mm-dd'T'HH:MM:ss",
	isoUtcDateTime: "UTC:yyyy-mm-dd'T'HH:MM:ss'Z'"
};

// Internationalization strings
dateFormat.i18n = {
	dayNames: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
	monthNames: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"]
};

// For convenience...
Date.prototype.format = function(mask, utc) {
	return dateFormat(this, mask, utc);
};

String.prototype.startsWith = function(p) {
	if (p === null) return false;

	var x = this.indexOf(p);
	if (x === 0) return true;
	else return false;
};

String.prototype.contains = function(p) {
	if (p === null) return false;

	var x = this.indexOf(p);
	if (x === -1) return false;
	else return true;
};

String.prototype.containsAll = function(p) {
	if (p === null) return false;

	var sz = p.length;

	for (var i = 0; i < sz; i++) {
		var a = p[i];
		var x = this.indexOf(a);
		if (x == -1) return false;
	}
	return true;
};

String.prototype.containsAny = function(p) {
	if (p === null) return false;

	var sz = p.length;

	for (var i = 0; i < sz; i++) {
		var a = p[i];
		var x = this.indexOf(a);
		if (x != -1) return true;
	}
	return false;
};

String.prototype.matches = function(p) {
	var rv = this.match(p);
	if (rv === null) return false;
	else return true;
}

String.prototype.endsWith = function(p) {
	if (p === null) return false;

	var x = this.lastIndexOf(p);
	if (x == this.length - p.length) return true;
	else return false;
}

Array.prototype.insertAt = function(o, index) {
	if (index > -1 && index <= this.length) {
		this.splice(index, 0, o);
		return true;
	}
	return false;
}

Array.prototype.insertBefore = function(toInsert, o) {
	var inserted = false;
	var index = this.indexOf(o);
	if (index == -1) return false;
	else {
		if (index == 0) {
			this.unshift(toInsert)
			return true;
		}
		else return this.insertAt(toInsert, index - 1);
	}
}

Array.prototype.insertAfter = function(toInsert, o) {
	var inserted = false;
	var index = this.indexOf(o);
	if (index == -1) return false;
	else {
		if (index == this.length - 1) {
			this.push(toInsert);
			return true;
		}
		else return this.insertAt(toInsert, index + 1);
	}
}

function nodePath(path) {
	var s = path.indexOf('/', path.indexOf("://") + 3);
	var e = path.indexOf('.');
	if (e > 0) return path.substr(s, e - s);
	if (s > 0) return path.substr(s);
	else return path;
}

function nodeBaseName(path) {
	var i = path.lastIndexOf('/');
	return path.substr(i + 1);
}

function nodeDirName(path) {
	var i = path.lastIndexOf('/');
	return path.substr(0, i);
}

function createUUID() {
	// http://www.ietf.org/rfc/rfc4122.txt
	var s = [];
	var hexDigits = "0123456789abcdef";
	for (var i = 0; i < 36; i++) {
		s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
	}
	s[14] = "4"; // bits 12-15 of the time_hi_and_version field to 0010
	s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
	s[8] = s[13] = s[18] = s[23] = "-";

	var uuid = s.join("");
	return uuid;
}
